/*
    Tic-Tac-Toe 68k v1.0
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 7/4/01; 1:40:50 PM

#include "ttt.h"

#include <graph.h>
#include <kbd.h>
#include <stdlib.h>
#include <mem.h>
#include <link.h>
#include <compat.h>

// draw the intro screen and wait for game to start or exit request
short int doIntro(void) {
	short int key = 0, done = FALSE;
	static short int mode = ONCALC;

	// clear the screen
	ClrScr();
	
	// draw the screen
	drawBanner(LOGO);
	drawBorder();
	drawCopyright();
	drawMenu();
	
	// draw the mode selector
	drawMode(mode);
	
	// wait until the user makes a valid selection
	while (!done) {
		// wait for keypress
		key = getKey(WAIT);
		
		// enter/esc = game request = play/quit
		if (key == KEY_ENTER || key == KEY_ESC) {
			done = TRUE;
		} else if (key == KEY_F1) {
			// toggle the mode
			drawMode(mode);
			
			// the last mode is LINKO, so wrap around
			if (mode == LINKO) {
				mode = ONCALC;
			} else {
				mode++;
			}
			
			// redraw the mode selector (XOR/XOR)
			drawMode(mode);
		}
	}

	// return either FALSE (exit request) or mode (play request)	
	return (key == KEY_ENTER) ? mode : FALSE;
}

// find the board position relative to pos in direction
short int findPos(short int pos, short int direction) {
	short int current = pos;
	
	if (direction == LEFT) {
		if (current == FIRST) {
			current = LAST;
		} else {
			current--;
		}
	} else if (direction == RIGHT) {
		if (current == LAST) {
			current = FIRST;
		} else {
			current++;
		}
	} else if (direction == UP) {
		if ((current / 3) == 0) {
			current = LAST - (2 - current);
		} else {
			current -= 3;
		}
	} else if (direction == DOWN) {
		if ((current / 3) == 2) {
			current %= 3;
		} else {
			current += 3;
		}
	}
	
	return current;
}

// play a move on the board (X or O)
void play(SQUARE *board, short int turn, short int move) {
	drawPiece(turn,board[move].pos);
	board[move].held = turn;
}

// check for game-over (winner or stalemate)
short int gameOver(SQUARE *board) {
	short int loop, p1, p2, p3;
	
	// chance for winner still exists...
	for (loop = 0; loop < 8; loop++) {
		p1 = winnings[loop][0];
		p2 = winnings[loop][1];
		p3 = winnings[loop][2];
			
		// if this is true, we have a winner
		if ((board[p1].held == board[p2].held) && (board[p2].held == board[p3].held)) {
			if (board[p1].held != EMPTY) {
				return board[p1].held;
			}
		}
	}
	
	// check for stalemate
	for (loop = FIRST; loop <= LAST; loop++) {
		// if empty position exists, game is not stalemate (yet)
		if (board[loop].held == EMPTY) {
			return -1;
		}
	}
	
	// no wins and no empty positions -> game is stalemate
	return EMPTY;
}

// play Tic-Tac-Toe game of type mode
void ticTacToe(short int mode) {
	SQUARE board[9];
	short int done = FALSE, turn = X, key, pos = 0;
	short int winner = -1, combo = rand() % WIN_COMBOS;

	// remove the intro-screen
	drawMenu();
	drawMode(mode);
	
	// create the Tic-Tac-Toe Board
	memcpy(board,start,9*sizeof(SQUARE));
	drawBoard();
	
	// draw the initial selector...
	drawSelector(board,turn,pos);
	
	while (!done) {
		// delete previous keypress
		key = 0;
	
		if (mode == ONCALC || (mode == VSCOMPUTER && !turn)) {
			// if it's player's turn, wait for keypress
			key = getKey(WAIT);
		} else if ((mode == LINKX && !turn) || (mode == LINKO && turn)) {
			// wait for keypress
			key = getKey(WAIT);
			
			// send keypress to link opponent
			LIO_SendData(&key,sizeof(short int));
		} else if ((mode == LINKX && turn) || (mode == LINKO && !turn)) {
			// wait for link keypress
			if (LIO_RecvData(&key,sizeof(short int),LINKWAIT) == LINKABORT) {
				// if user aborts the link option, abort the game
				return;
			} else {
				if (linkmode == TRANSLATE) {
					// translate keypresses from opposite calc-type
					key = translateKey(key);
				}
			}
		} else if (mode == VSCOMPUTER && turn) {
			// play the AI's turn
			combo = aiPlay(board,combo);
			turn = !turn;
			
			if ((winner = gameOver(board)) != -1) {
				done = TRUE;
			}
			
			// recreate the selector
			drawSelector(board,turn,pos);
		}
		
		if (key == KEY_ESC) {
			// quit key
			done = TRUE;
		} else if (key == KEY_ENTER) {
			if (board[pos].held == EMPTY) {
				// remove old selector
				drawSelector(board,turn,pos);
			
				// place the new square holder
				play(board,turn,pos);
			
				// switch the turn
				turn = !turn;
			
				if ((winner = gameOver(board)) == -1) {
					if (mode != VSCOMPUTER) {
						// redraw the selector
						pos = findPos(pos,RIGHT);
						drawSelector(board,turn,pos);
					}
				} else {
					done = TRUE;
				}
			}
		} else if (key == KEY_LEFT) {
			// move selector left
			drawSelector(board,turn,pos);
			pos = findPos(pos,LEFT);
			drawSelector(board,turn,pos);
		} else if (key == KEY_RIGHT) {
			// move selector right
			drawSelector(board,turn,pos);
			pos = findPos(pos,RIGHT);
			drawSelector(board,turn,pos);
		} else if (key == KEY_UP) {
			// move selector up
			drawSelector(board,turn,pos);
			pos = findPos(pos,UP);
			drawSelector(board,turn,pos);
		} else if (key == KEY_DOWN) {
			// move selector down
			drawSelector(board,turn,pos);
			pos = findPos(pos,DOWN);
			drawSelector(board,turn,pos);
		}
	}
	
	if (winner != -1) {
		// erase the logo
		drawBanner(LOGO);
		
		// draw the winner banner
		drawBanner(winner);
		
		// wait for keypress
		getKey(WAIT);
	}
}
